@@ -4,16 +4,26 @@ Contains config and settings for the repo.
 
 import importlib
 import json
-import os
-import warnings
 import logging
+import os
+import time
+import warnings
+from contextlib import ContextDecorator
 from typing import Tuple, Optional
 
-import boto
-from boto import sqs
-
+import boto3
+import botocore.exceptions as boto_exceptions
 from PIL import Image
 
+# Configure a simple logger that works with AWS cloudwatch
+if len(logging.getLogger().handlers) > 0:
+    # The Lambda environment pre-configures a handler logging to stderr.
+    # If a handler is already configured,
+    # `.basicConfig` does not execute. Thus we set the level directly.
+    logging.getLogger().setLevel(logging.INFO)
+else:
+    logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s')
+
 
 def filter_warnings():
     """ Filters out some verified warnings. """
@@ -64,7 +74,11 @@ def get_s3_conn():
     - If not found there it will default to credentials in ~/.aws/credentials
     """
     aws_key_id, aws_key_secret = get_aws_credentials()
-    return boto.connect_s3(aws_key_id, aws_key_secret)
+
+    return boto3.resource('s3',
+                          region_name="us-west-2",
+                          aws_access_key_id=aws_key_id,
+                          aws_secret_access_key=aws_key_secret)
 
 
 def get_sqs_conn():
@@ -75,10 +89,10 @@ def get_sqs_conn():
     - If not found there it will default to credentials in ~/.aws/credentials
     """
     aws_key_id, aws_key_secret = get_aws_credentials()
-    return sqs.connect_to_region(
-        "us-west-2",
-        aws_access_key_id=aws_key_id,
-        aws_secret_access_key=aws_key_secret)
+    return boto3.resource('sqs',
+                          region_name="us-west-2",
+                          aws_access_key_id=aws_key_id,
+                          aws_secret_access_key=aws_key_secret)
 
 
 def get_local_model_path():
@@ -88,6 +102,20 @@ def get_local_model_path():
     return local_model_path
 
 
+class log_entry_and_exit(ContextDecorator):
+    def __init__(self, name):
+        self.name = name
+        self.start_time = None
+
+    def __enter__(self):
+        self.start_time = time.time()
+        logging.info('Entering: %s', self.name)
+
+    def __exit__(self, exc_type, exc, exc_tb):
+        logging.info('Exiting: %s after %f seconds.', self.name,
+                     time.time() - self.start_time)
+
+
 LOCAL_MODEL_PATH = get_local_model_path()
 
 HAS_LOCAL_MODEL_PATH = LOCAL_MODEL_PATH is not None and \
@@ -106,11 +134,24 @@ FEATURE_EXTRACTOR_NAMES = [
     'efficientnet_b0_ver1'
 ]
 
+MODEL_WEIGHTS_SHA = {
+    'vgg16':
+        'fb83781de0e207ded23bd42d7eb6e75c1e915a6fbef74120f72732984e227cca',
+    'efficientnet-b0':
+        'c3dc6d304179c6729c0a0b3d4e60c728bdcf0d82687deeba54af71827467204c',
+}
+
+CLASSIFIER_TYPES = [
+    'LR',
+    'MLP'
+]
+
 TRAINER_NAMES = [
     'minibatch'
 ]
 
 MODELS_BUCKET = 'spacer-tools'
+TEST_BUCKET = 'spacer-test'
 
 STORAGE_TYPES = [
     's3',
@@ -131,32 +172,24 @@ MIN_TRAINIMAGES = 10
 # Check access to select which tests to run.
 HAS_CAFFE = importlib.util.find_spec("caffe") is not None
 
+
 try:
-    conn = get_s3_conn()
+    s3 = get_s3_conn()
+    s3.meta.client.head_bucket(Bucket=TEST_BUCKET)
     HAS_S3_TEST_ACCESS = True
-    HAS_S3_MODEL_ACCESS = True
-except boto.exception.NoAuthHandlerFound:
-    conn = None
+except [boto_exceptions.ClientError, boto_exceptions.NoCredentialsError]:  # pragma: no cover
+    logging.info("No connection to spacer-test bucket, "
+                 "can't run remote tests")
     HAS_S3_TEST_ACCESS = False
+
+try:
+    s3 = get_s3_conn()
+    s3.meta.client.head_bucket(Bucket=MODELS_BUCKET)
+    HAS_S3_MODEL_ACCESS = True
+except [boto_exceptions.ClientError, boto_exceptions.NoCredentialsError]:  # pragma: no cover
+    logging.info("No connection to spacer-tools bucket, "
+                 "can't run remote tests")
     HAS_S3_MODEL_ACCESS = False
-    logging.info("-> No s3 connection.")
-
-
-if HAS_S3_TEST_ACCESS:
-    try:
-        bucket = conn.get_bucket('spacer-test', validate=True)
-    except boto.exception.S3ResponseError:  # pragma: no cover
-        logging.info("-> No connection to spacer-test bucket, "
-                     "can't run remote tests")
-        HAS_S3_TEST_ACCESS = False
-
-if HAS_S3_MODEL_ACCESS:
-    try:
-        bucket = conn.get_bucket('spacer-tools', validate=True)
-    except boto.exception.S3ResponseError:  # pragma: no cover
-        logging.info("-> No connection to spacer-tools bucket, "
-                     "can't download models")
-        HAS_S3_MODEL_ACCESS = False
 
 # Add margin to avoid warnings when running unit-test.
 Image.MAX_IMAGE_PIXELS = MAX_IMAGE_PIXELS + 20000
