@@ -7,7 +7,7 @@ python-native data-structures such that it can be stored.
 from typing import List, Tuple, Dict, Union, Optional
 
 from spacer import config
-from spacer.data_classes import DataClass
+from spacer.data_classes import DataClass, ImageLabels
 
 
 class DataLocation(DataClass):
@@ -120,9 +120,10 @@ class TrainClassifierMsg(DataClass):
                  job_token: str,  # Job token, for caller reference.
                  trainer_name: str,  # Name of trainer to use.
                  nbr_epochs: int,  # Number of epochs to do training.
-                 traindata_loc: DataLocation,  # Traindata
-                 valdata_loc: DataLocation,  # Valdata
-                 features_loc: DataLocation,  # Location of features. Key is set from traindata and valdata during training.
+                 clf_type: str,  # Name of classifier to use.
+                 train_labels: ImageLabels,  # Traindata
+                 val_labels: ImageLabels,  # Valdata
+                 features_loc: DataLocation,  # Location of features. Key is set from train and val labels during data load.
                  previous_model_locs: List[DataLocation],  # Previous models to be evaluated on the valdata.
                  model_loc: DataLocation,  # Where to store model.
                  valresult_loc: DataLocation,  # Model result on val.
@@ -133,8 +134,9 @@ class TrainClassifierMsg(DataClass):
         self.job_token = job_token
         self.trainer_name = trainer_name
         self.nbr_epochs = nbr_epochs
-        self.traindata_loc = traindata_loc
-        self.valdata_loc = valdata_loc
+        self.clf_type = clf_type
+        self.train_labels = train_labels
+        self.val_labels = val_labels
         self.features_loc = features_loc
         self.previous_model_locs = previous_model_locs
         self.model_loc = model_loc
@@ -146,8 +148,9 @@ class TrainClassifierMsg(DataClass):
             job_token='123_abc',
             trainer_name='minibatch',
             nbr_epochs=2,
-            traindata_loc=DataLocation('memory', 'my_traindata.json'),
-            valdata_loc=DataLocation('memory', 'my_valdata.json'),
+            clf_type='MLP',
+            train_labels=ImageLabels.example(),
+            val_labels=ImageLabels.example(),
             features_loc=DataLocation('memory', ''),
             previous_model_locs=[
                 DataLocation('memory', 'previous_model1.pkl'),
@@ -162,8 +165,9 @@ class TrainClassifierMsg(DataClass):
             'job_token': self.job_token,
             'trainer_name': self.trainer_name,
             'nbr_epochs': self.nbr_epochs,
-            'traindata_loc': self.traindata_loc.serialize(),
-            'valdata_loc': self.valdata_loc.serialize(),
+            'clf_type': self.clf_type,
+            'train_labels': self.train_labels.serialize(),
+            'val_labels': self.val_labels.serialize(),
             'features_loc': self.features_loc.serialize(),
             'previous_model_locs': [entry.serialize()
                                     for entry in self.previous_model_locs],
@@ -178,8 +182,9 @@ class TrainClassifierMsg(DataClass):
             job_token=data['job_token'],
             trainer_name=data['trainer_name'],
             nbr_epochs=data['nbr_epochs'],
-            traindata_loc=DataLocation.deserialize(data['traindata_loc']),
-            valdata_loc=DataLocation.deserialize(data['valdata_loc']),
+            clf_type=data['clf_type'],
+            train_labels=ImageLabels.deserialize(data['train_labels']),
+            val_labels=ImageLabels.deserialize(data['val_labels']),
             features_loc=DataLocation.deserialize(data['features_loc']),
             previous_model_locs=[DataLocation.deserialize(entry)
                                  for entry in data['previous_model_locs']],
